/*
 * Copyright (C) 2020, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "popupitemwidget.h"
#include <QPainterPath>
#include <QTranslator>
#include <QDomDocument>
#include <QXmlStreamReader>
#include <QDBusMessage>
#include <QDBusConnection>
#include <QDBusPendingCallWatcher>

popupItemWidget::popupItemWidget(QWidget *parent, notifyReceiveInfo *entryInfo)
                                : QWidget(parent)
                                , m_pentryInfo(entryInfo)
                                , m_pIconLabel(new QLabel)
                                , m_pSummaryLabel(new QLabel)
                                , m_pTextBodyLabel(new QLabel)
                                , m_pCloseButton(new QPushButton)
                                , m_pListButton(new QList<QPushButton *>)
{
    if(PLATFORM::g_platformType == PLATFORM::Intel){
        QTranslator *translator = new QTranslator;
        if (translator->load(QLocale(), QLatin1String("ukui-notification-daemon"), QLatin1String("_"), QLatin1String("/usr/share/ukui-notification-daemon")))
            QApplication::installTranslator(translator);
        else
            qWarning() << "cannot load translator ukui-notification-daemon_" << QLocale::system().name() << ".qm!";
    }

    /* 设置窗口属性 */
    setWidgetAttribute();

    /* 初始化label信息 */
    initLabelSizeInfo();

    /* 初始化操作按钮 */
    initOperationButton();

    /* 初始化图标 */
    initIconWidgetlayout();

    /* 初始化中间部分UI */
    initInfoWidgetLayout();

    /* 初始化关闭按钮 */
    initCloseButtonWidget();

    /* 初始化整体UI */
    initUiLayout();

    /* 初始化定时器 */
    initTimer();

    /* 初始化动画 */
    initWidgetAnimations();

    if(PLATFORM::g_platformType == PLATFORM::V101){
        /* 初始化控制面板透明度gsetting值 */
        initTransparencySetting();
    }
    /* 初始化信号连接 */
    //connect(this, &popupItemWidget::actionButtonClicked, this, &popupItemWidget::onActionButtonClicked);
}

/* 设置显示一行数据 */
void popupItemWidget::setEntryData(notifyReceiveInfo *entryInfo)
{
    if (nullptr == entryInfo) {
        qWarning() << "设置条目中信息数据传入值为Nullptr";
        return;
    }
    m_pentryInfo = entryInfo;
    m_poutTimer->stop();
    convertToImage(m_pentryInfo->appIcon());
    judgeBodyExsit();
    setWidgetDate();
    judgeActionExsit();
    m_poutTimer->start();
    return;
}

/* 超时信息收纳到侧边栏 */
void popupItemWidget::stowToSidebar()
{
    QDBusMessage msg = QDBusMessage::createMethodCall("org.ukui.Sidebar",
                                                    "/org/ukui/Sidebar/notification",
                                                    "org.ukui.Sidebar.notification",
                                                    "sidebarNotification");
    //发送的数据：appName, appIcon, summary, body, urlStr,  actions
    msg << m_pentryInfo->appName()
        << m_pentryInfo->appIcon()
        << m_pentryInfo->summary()
        << m_pentryInfo->body()
        << m_pentryInfo->urlStr()
        << m_pentryInfo->actionStr();  //填充要发送的数据
    qInfo()<<"Stow to sidebar:"<<msg;
    //调用method
    QDBusPendingCall async = QDBusConnection::sessionBus().asyncCall(msg);
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(async, this);
}

/* 初始化整体UI布局 */
void popupItemWidget::initUiLayout()
{
    if(PLATFORM::g_platformType==PLATFORM::V101){
        this->setContentsMargins(0, 0, 0, 0);
        m_pMainHBoxLayout = new QHBoxLayout();
        m_pMainHBoxLayout->setContentsMargins(0, 0, 0, 0);
        m_pMainHBoxLayout->setSpacing(0);

        m_pMainHBoxLayout->addWidget(m_pIconWidget);
        m_pMainHBoxLayout->addItem(new QSpacerItem(12, 0));
        m_pMainHBoxLayout->addWidget(m_pInfoAreaWidget);
        m_pMainHBoxLayout->addWidget(m_pCloseButtonWidget);
        this->setFixedSize(372, 134);
        this->setLayout(m_pMainHBoxLayout);
    }
    else if(PLATFORM::g_platformType==PLATFORM::Intel){
        this->setContentsMargins(0, 0, 0, 0);
        m_pMainHBoxLayout = new QHBoxLayout();
        m_pMainHBoxLayout->setContentsMargins(0, 0, 0, 0);

        m_pMainHBoxLayout->addItem(new QSpacerItem(16, 0, QSizePolicy::Fixed));
        m_pMainHBoxLayout->addWidget(m_pIconWidget);
        m_pMainHBoxLayout->addItem(new QSpacerItem(2, 0, QSizePolicy::Fixed));
        m_pMainHBoxLayout->addWidget(m_pInfoAreaWidget);
        m_pMainHBoxLayout->addItem(new QSpacerItem(2, 0, QSizePolicy::Fixed));
        m_pMainHBoxLayout->addWidget(m_pCloseButtonWidget);
        m_pMainHBoxLayout->addItem(new QSpacerItem(10, 0, QSizePolicy::Fixed));
        this->setFixedSize(360, 156);
        this->setLayout(m_pMainHBoxLayout);
    }
}

void popupItemWidget::initIconWidgetlayout()
{
    if(PLATFORM::g_platformType==PLATFORM::V101){
        m_pIconWidget = new QWidget();
        m_pIconWidget->setFixedWidth(28);
        m_pIconWidget->setContentsMargins(0, 0, 0, 0);
        m_pIconWidgetLayout = new QVBoxLayout();
        m_pIconWidgetLayout->setContentsMargins(12, 0, 0, 0);
        m_pIconWidgetLayout->setSpacing(0);
        /* 初始化图标label大小 */
        m_pIconLabel->setFixedSize(16, 16);
        m_pIconWidgetLayout->addItem(new QSpacerItem(10, 18));
        m_pIconWidgetLayout->addWidget(m_pIconLabel);
        m_pIconWidgetLayout->addItem(new QSpacerItem(10, 102, QSizePolicy::Expanding));
        m_pIconWidget->setLayout(m_pIconWidgetLayout);
    }
    else if(PLATFORM::g_platformType==PLATFORM::Intel){
        m_pIconWidget = new QWidget();
        m_pIconWidget->setFixedWidth(16);
        m_pIconWidget->setContentsMargins(0, 0, 0, 0);
        m_pIconWidgetLayout = new QVBoxLayout();
        m_pIconWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pIconWidgetLayout->setSpacing(0);
        /* 初始化图标label大小 */
        m_pIconLabel->setFixedSize(16, 16);
        m_pIconWidgetLayout->addItem(new QSpacerItem(10, 10));
        m_pIconWidgetLayout->addWidget(m_pIconLabel);
        m_pIconWidgetLayout->addItem(new QSpacerItem(10, 130, QSizePolicy::Expanding));
        m_pIconWidget->setLayout(m_pIconWidgetLayout);
    }
}

/* 初始化中间部分布局 */
void popupItemWidget::initInfoWidgetLayout()
{
    if(PLATFORM::g_platformType==PLATFORM::V101){
        m_pInfoAreaWidget = new QWidget();
        m_pInfoAreaWidget->setContentsMargins(0, 0, 0, 0);
        m_pLeftVBoxLayout = new QVBoxLayout();
        m_pLeftVBoxLayout->setContentsMargins(0, 0, 0, 0);
        m_pLeftVBoxLayout->setSpacing(0);

        m_pLeftVBoxLayout->addWidget(m_pSummaryLabelWidget);
        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 0, QSizePolicy::Minimum));
        m_pLeftVBoxLayout->addWidget(m_pBodyLabelWidget);
        m_pLeftVBoxLayout->addWidget(m_pOperationWidget);
        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 5, QSizePolicy::Expanding));
        m_pInfoAreaWidget->setLayout(m_pLeftVBoxLayout);
    }
    else if(PLATFORM::g_platformType==PLATFORM::Intel){
        m_pInfoAreaWidget = new QWidget();
        m_pInfoAreaWidget->setContentsMargins(0, 0, 0, 0);
        m_pLeftVBoxLayout = new QVBoxLayout();
        m_pLeftVBoxLayout->setContentsMargins(0, 0, 0, 0);
        m_pLeftVBoxLayout->setSpacing(0);

        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 10, QSizePolicy::Fixed));
        m_pLeftVBoxLayout->addWidget(m_pSummaryLabelWidget);
        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 0, QSizePolicy::Minimum));
        m_pLeftVBoxLayout->addWidget(m_pBodyLabelWidget);
        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 5, QSizePolicy::Minimum));
        m_pLeftVBoxLayout->addWidget(m_pOperationWidget);
        m_pLeftVBoxLayout->addItem(new QSpacerItem(10, 10, QSizePolicy::Minimum));
        m_pInfoAreaWidget->setLayout(m_pLeftVBoxLayout);
    }
}

/* 初始化关闭按钮 */
void popupItemWidget::initCloseButtonWidget()
{
    if(PLATFORM::g_platformType==PLATFORM::V101){
        m_pCloseButtonWidget = new QWidget();
        m_pCloseButtonWidget->setFixedWidth(34);
        m_pCloseButtonWidget->setContentsMargins(0, 0, 0, 0);
        m_pCloseWidgetLayout = new QVBoxLayout();
        m_pCloseWidgetLayout->setSpacing(0);
        m_pCloseWidgetLayout->setContentsMargins(0, 4, 4, 0);

        QIcon closeButtonIcon = QIcon::fromTheme("window-close-symbolic");
        m_pCloseButton->setProperty("isWindowButton", 0x2);
        m_pCloseButton->setProperty("useIconHighlightEffect", 0x8);
        m_pCloseButton->setFlat(true);
        m_pCloseButton->setIcon(closeButtonIcon);
        m_pCloseButton->setIconSize(QSize(16, 16));
        m_pCloseButton->setFixedSize(30, 30);
        connect(m_pCloseButton, &QPushButton::clicked, this, &popupItemWidget::closeButtonSlots);
    //    m_pCloseWidgetLayout->addItem(new QSpacerItem(10, 12));
        m_pCloseWidgetLayout->addWidget(m_pCloseButton);
        m_pCloseWidgetLayout->addItem(new QSpacerItem(10, 110, QSizePolicy::Expanding));
        m_pCloseButtonWidget->setLayout(m_pCloseWidgetLayout);
    }
    else if(PLATFORM::g_platformType==PLATFORM::Intel){
        m_pCloseButtonWidget = new QWidget();
        m_pCloseButtonWidget->setFixedWidth(16);
        m_pCloseButtonWidget->setContentsMargins(0, 0, 0, 0);
        m_pCloseWidgetLayout = new QVBoxLayout();
        m_pCloseWidgetLayout->setSpacing(0);
        m_pCloseWidgetLayout->setContentsMargins(0, 0, 0, 0);

        QIcon closeButtonIcon = QIcon::fromTheme("window-close-symbolic");
        m_pCloseButton->setIcon(closeButtonIcon);
        m_pCloseButton->setIconSize(QSize(12, 12));
        m_pCloseButton->setFixedSize(16, 16);
        connect(m_pCloseButton, &QPushButton::clicked, this, &popupItemWidget::closeButtonSlots);
        m_pCloseWidgetLayout->addItem(new QSpacerItem(10, 10));
        if(m_pentryInfo->timeout() != "0"){  //强通知消息不显示关闭图标
            m_pCloseWidgetLayout->addWidget(m_pCloseButton);
        }
        m_pCloseWidgetLayout->addItem(new QSpacerItem(10, 130, QSizePolicy::Expanding));
        m_pCloseButtonWidget->setLayout(m_pCloseWidgetLayout);
    }
}

void popupItemWidget::initLabelSizeInfo()
{
    const QByteArray id(UKUI_STYLE_SETTING_PATH);
    if (QGSettings::isSchemaInstalled(id)) {
        m_pFontStyleGsetting = new QGSettings(id);
        if(m_pFontStyleGsetting->keys().contains(UKUI_STYLE_FONT_SIZE_SETTING_KEY)){
            m_iStyleFontSize = m_pFontStyleGsetting->get(UKUI_STYLE_FONT_SIZE_SETTING_KEY).toInt();
        }
    }

    if(PLATFORM::g_platformType==PLATFORM::V101){
        /* 存放标题Label */
        m_pSummaryLabelWidget = new QWidget();
        m_pSummaryLabelWidget->setContentsMargins(0, 0, 0, 0);
        //m_pSummaryLabelWidget->setStyleSheet("QWidget{border: 1px solid rgba(255,0,0,1);}");
        m_pSummaryLabelWidgetLayout = new QVBoxLayout();
        m_pSummaryLabelWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pSummaryLabelWidgetLayout->setSpacing(0);
        m_pSummaryLabel->setFixedWidth(300);
        m_pSummaryLabel->setAlignment(Qt::AlignVCenter);

        /* 存放文本信息Label */
        m_pBodyLabelWidget = new QWidget();
        m_pBodyLabelWidget->adjustSize();
        m_pBodyLabelWidget->setContentsMargins(0, 0, 0, 0);
        //m_pBodyLabelWidget->setStyleSheet("QWidget{border: 1px solid rgba(255,0,0,1);}");
        m_pBodyLabelWidgetLayout = new QVBoxLayout();
        m_pBodyLabelWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pBodyLabelWidgetLayout->setSpacing(0);
        m_pTextBodyLabel->setFixedWidth(300);
        m_pTextBodyLabel->setAlignment(Qt::AlignVCenter);
        // 根据字体大小来设置间距改变布局
        //setWidgetFontSpace();
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);

        m_pSummaryLabelWidget->setLayout(m_pSummaryLabelWidgetLayout);
        m_pBodyLabelWidget->setLayout(m_pBodyLabelWidgetLayout);
    }
    else if(PLATFORM::g_platformType==PLATFORM::Intel){
        /* 存放标题Label */
        m_pSummaryLabelWidget = new QWidget();
        m_pSummaryLabelWidget->setContentsMargins(0, 0, 0, 0);
        m_pSummaryLabelWidgetLayout = new QVBoxLayout();
        m_pSummaryLabelWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pSummaryLabelWidgetLayout->setSpacing(0);
        QTimer::singleShot(1, m_pSummaryLabel, [=]() {
            QFont summaryFont = m_pSummaryLabel->font();
            summaryFont.setPixelSize(16);
            summaryFont.setFamily("Noto Sans CJK SC");
        });
        m_pSummaryLabel->setFixedWidth(300);
        m_pSummaryLabel->setAlignment(Qt::AlignVCenter);
        QLabel * m_label = new QLabel(this);
        m_label->setText(tr("Tips"));

        m_pSummaryLabelWidgetLayout->addWidget(m_label);
        m_pSummaryLabelWidgetLayout->addItem(new QSpacerItem(10, 0));
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);
        m_pSummaryLabelWidget->setLayout(m_pSummaryLabelWidgetLayout);

        /* 存放文本信息Label */
        m_pBodyLabelWidget = new QWidget();
        m_pBodyLabelWidget->setContentsMargins(0, 0, 0, 0);

        m_pBodyLabelWidgetLayout = new QVBoxLayout();
        m_pBodyLabelWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pBodyLabelWidgetLayout->setSpacing(0);
        QTimer::singleShot(1, m_pTextBodyLabel, [=]() {
            QFont bodyFont = m_pTextBodyLabel->font();
            bodyFont.setPixelSize(14);
            m_pTextBodyLabel->setFont(bodyFont);
        });
        m_pTextBodyLabel->setFixedWidth(300);
        m_pTextBodyLabel->setAlignment(Qt::AlignVCenter);
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);
        m_pBodyLabelWidget->setLayout(m_pBodyLabelWidgetLayout);
    }
}

void popupItemWidget::initOperationButton()
{
    if(PLATFORM::g_platformType == PLATFORM::V101){
        m_pOperationWidget = new QWidget();
        m_pOperationWidget->setContentsMargins(0, 0, 0, 0);
        m_pOperationWidget->setFixedHeight(49);
        m_pOperationButtonWidgetLayout = new QHBoxLayout();
        m_pOperationButtonWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pOperationButtonWidgetLayout->setSpacing(0);
        m_pOperationButtonWidgetLayout->addItem(new QSpacerItem(372, 20, QSizePolicy::Expanding));
        m_pOperationWidget->setLayout(m_pOperationButtonWidgetLayout);
    }
    else if(PLATFORM::g_platformType == PLATFORM::Intel){
        m_pOperationWidget = new QWidget();
        m_pOperationWidget->setContentsMargins(0, 0, 0, 0);
        m_pOperationButtonWidgetLayout = new QHBoxLayout();
        m_pOperationButtonWidgetLayout->setContentsMargins(0, 0, 0, 0);
        m_pOperationButtonWidgetLayout->setSpacing(0);
        m_pOperationButtonWidgetLayout->addItem(new QSpacerItem(372, 20, QSizePolicy::Expanding));
        m_pOperationWidget->setLayout(m_pOperationButtonWidgetLayout);
    }
}

/* 初始化显示时长定时器 */
void popupItemWidget::initTimer()
{
    m_poutTimer = new QTimer();
    m_poutTimer->setSingleShot(true);
    connect(m_poutTimer, &QTimer::timeout, this, &popupItemWidget::ShowTimeoutSlots);

    m_quitTimer = new QTimer();
    m_quitTimer->setInterval(60 * 1000);
    m_quitTimer->setSingleShot(true);
    connect(m_quitTimer, &QTimer::timeout, this, &popupItemWidget::qiutAppTimerSlots);
}

/* 初始化控制面板透明度gsetting值 */
void popupItemWidget::initTransparencySetting()
{
    QByteArray id(UKUI_TRANSPARENCY_SETTING_PATH);
    if (QGSettings::isSchemaInstalled(id)) {
        m_pTransparencyGsetting = new QGSettings(id);
        if (m_pTransparencyGsetting->keys().contains(UKUI_TRANSPARENCY_SETTING_KEY)) {
            m_fTransparencyValue = m_pTransparencyGsetting->get(UKUI_TRANSPARENCY_SETTING_KEY).toDouble();
        }
        connect(m_pTransparencyGsetting, &QGSettings::changed, this, [=](QString key) {
           if (key == UKUI_TRANSPARENCY_SETTING_KEY) {
               m_fTransparencyValue = m_pTransparencyGsetting->get(UKUI_TRANSPARENCY_SETTING_KEY).toDouble();
               update();
           }
        });
    }
    return;
}

/* 设置窗口属性， 无边框， 任务栏无图标 */
void popupItemWidget::setWidgetAttribute()
{
    setWindowFlags(Qt::FramelessWindowHint | Qt::X11BypassWindowManagerHint);
    setAttribute(Qt::WA_TranslucentBackground);
    return;
}

/* 初始化显示和消失动画 */
void popupItemWidget::initWidgetAnimations()
{
    m_pOutAnimation = new QPropertyAnimation(this, "geometry");
    connect(m_pOutAnimation, &QPropertyAnimation::finished, this, &popupItemWidget::OutAnimationFinishSlots);
    connect(m_pOutAnimation, &QPropertyAnimation::valueChanged, this, [=](QVariant value) {
        if (m_bActionSignals) {
            emit animationAction(value, this);
            m_bActionSignals = false;
        }
    });

    m_pOutAnimation->setDuration(200);
    m_pOutAnimation->setEasingCurve(QEasingCurve::OutCubic);

    m_pMoveAnimation = new QPropertyAnimation(this, "geometry");
    m_pMoveAnimation->setDuration(300);
    m_pMoveAnimation->setEasingCurve(QEasingCurve::OutCubic);
    m_pMoveAnimation->setStartValue(QRect(this->width() - 1, 0, this->width(), 60));
    m_pMoveAnimation->setEndValue(QRect(0, 0, this->width(), 60));
}

/*
 * 根据字体大小设置控件之间的间距
 * 暂时不用，后期可参考此函数做细致优化
*/
void popupItemWidget::setWidgetFontSpace()
{
//    QTimer::singleShot(1, m_pSummaryLabel, [=]() {
//        QFont summaryFont = m_pSummaryLabel->font();
//        summaryFont.setPointSizeF(m_iStyleFontSize * (16.0/14.0));
//        summaryFont.setFamily("Noto Sans CJK SC");
//        m_pSummaryLabel->setFont(summaryFont);
//    });
    if (m_iStyleFontSize == 10 || m_iStyleFontSize == 11 || m_iStyleFontSize == 12) {
        m_pSummaryLabelWidget->setFixedHeight(34);
        m_pSummaryLabel->setFixedHeight(26);

        m_pSummaryLabelWidgetLayout->addItem(new QSpacerItem(10, 8, QSizePolicy::Expanding));
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);

        m_pBodyLabelWidgetLayout->addItem(new QSpacerItem(10, 8, QSizePolicy::Expanding));
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);
    }  else if (m_iStyleFontSize == 13) {
        m_pSummaryLabelWidget->setFixedHeight(36);
        m_pSummaryLabel->setFixedHeight(28);
        m_pSummaryLabelWidgetLayout->addItem(new QSpacerItem(10, 8));
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);

        m_pBodyLabelWidgetLayout->addItem(new QSpacerItem(10, 8, QSizePolicy::Expanding));
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);
    } else if (m_iStyleFontSize == 14) {
        m_pSummaryLabelWidget->setFixedHeight(38);
        m_pSummaryLabel->setFixedHeight(30);
        m_pSummaryLabelWidgetLayout->addItem(new QSpacerItem(10, 8));
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);

        m_pBodyLabelWidgetLayout->addItem(new QSpacerItem(10, 6, QSizePolicy::Expanding));
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);
    } else if (m_iStyleFontSize == 15 || m_iStyleFontSize == 16) {
        m_pSummaryLabelWidget->setFixedHeight(40);
        m_pSummaryLabel->setFixedHeight(34);
        m_pSummaryLabelWidgetLayout->addItem(new QSpacerItem(10, 6));
        m_pSummaryLabelWidgetLayout->addWidget(m_pSummaryLabel);
        m_pBodyLabelWidgetLayout->addItem(new QSpacerItem(10, 1, QSizePolicy::Expanding));
        m_pBodyLabelWidgetLayout->addWidget(m_pTextBodyLabel);
    }
    delete m_pFontStyleGsetting;
    m_pFontStyleGsetting = nullptr;
    return;
}

/* 设置显示数据文本 */
void popupItemWidget::setWidgetDate()
{
    m_pTextBodyLabel->setText(SetFormatBody(m_pentryInfo->body(), m_pTextBodyLabel));
    m_pSummaryLabel->setText(SetFormatBody(m_pentryInfo->summary(), m_pSummaryLabel));
    connect(m_pFontStyleGsetting, &QGSettings::changed, this, [=](QString key) {
       if (key == "systemFontSize") {
           m_pTextBodyLabel->setText(SetFormatBody(m_pentryInfo->body(), m_pTextBodyLabel));
           m_pSummaryLabel->setText(SetFormatBody(m_pentryInfo->summary(), m_pSummaryLabel));
       }
    });
    return;
}
/* 替换弹窗的内容 */
void popupItemWidget::updateWidgetDate(notifyReceiveInfo *replaceInfo)
{
    m_pentryInfo = replaceInfo;
    m_pentryInfo->setId(replaceInfo->replacesId());
    //替换主主题和正文
    disconnect(this,&popupItemWidget::mouseMissed,this,0);
    judgeBodyExsit();
    setWidgetDate();
    //替换按钮动作
    disconnect(this,&popupItemWidget::actionInvokedMissed,this,0);
    judgeActionExsit();
    return;
}

bool popupItemWidget::containsMouse() const
{
    QRect rectToGlobal = QRect(mapToGlobal(rect().topLeft()),
                               mapToGlobal(rect().bottomRight()));
    return rectToGlobal.contains(QCursor::pos());
}

/* 通知图标转换 */
void popupItemWidget::convertToImage(QString iconPath)
{
    const qreal pixelRatio = qApp->primaryScreen()->devicePixelRatio();
    QPixmap pixmap;

    if(PLATFORM::g_platformType == PLATFORM::V101){
        if (iconPath.startsWith("data:image/")) {
            QStringList strs = iconPath.split("base64,");
            if (strs.length() == 2) {
                QByteArray data = QByteArray::fromBase64(strs.at(1).toLatin1());
                pixmap.loadFromData(data);
            }
        }
    }

    pixmap = QIcon::fromTheme(iconPath).pixmap(QSize(45, 45));
    if (pixmap.isNull()) {
        QString iconUrl;
        const QUrl url(iconPath);
        iconUrl = url.isLocalFile() ? url.toLocalFile() : url.url();

        if(PLATFORM::g_platformType == PLATFORM::V101){
            if (iconPath.contains("file://")) {
                int length = iconPath.length();
                iconPath = iconPath.mid(7, length);
            }
            const QIcon &icon = QIcon::fromTheme(iconPath, QIcon::fromTheme("application-x-desktop"));
            pixmap = icon.pixmap(width() * pixelRatio, height() * pixelRatio);
        }
        else if(PLATFORM::g_platformType == PLATFORM::Intel){
            const QIcon &icon = QIcon::fromTheme(iconPath, QIcon::fromTheme("dialog-information"));
            pixmap = icon.pixmap(QSize(45, 45));
        }
    }

    if (!pixmap.isNull()) {
        pixmap = pixmap.scaled(m_pIconLabel->width() * pixelRatio, m_pIconLabel->height() * pixelRatio,
                               Qt::KeepAspectRatioByExpanding,
                               Qt::SmoothTransformation);

        pixmap.setDevicePixelRatio(pixelRatio);
    }

    m_pIconLabel->setPixmap(pixmap);
    return;
}

/* 设置...字样 */
QString popupItemWidget::SetFormatBody(QString text, QLabel *label)
{
    QFontMetrics fontMetrics(label->font());
    if(PLATFORM::g_platformType == PLATFORM::V101){
        QFontMetrics fontMetricsTmp(label->font());
        fontMetrics = fontMetricsTmp;
    }
    else if(PLATFORM::g_platformType == PLATFORM::Intel){ //Intel固定14号字体
        QFont bodyFont;
        bodyFont.setPixelSize(14);
        QFontMetrics fontMetricsTmp(bodyFont);
        fontMetrics = fontMetricsTmp;
    }

    int LableWidth = label->width();
    int fontSize = fontMetrics.width(text);
    QString formatBody = text;
    if(fontSize > (LableWidth - 10)) {
        QStringList list = formatBody.split("\n");
        if(PLATFORM::g_platformType == PLATFORM::V101){  //V10sp1 通知显示1行
            if (list.size() >= 2) {
                //当有几行时，只需要截取第一行就行，在第一行后面加...
                // 判断第一行是否是空行
                formatBody = judgeBlankLine(list);
                formatBody = formatBody + "aa";
                int oneFontSize = fontMetrics.width(formatBody);
                if (oneFontSize > (LableWidth - 10)) {
                    formatBody = fontMetrics.elidedText(formatBody, Qt::ElideRight, LableWidth - 10);
                    return formatBody;
                } else {
                    if (!substringSposition(formatBody, list)) {
                        int oneFontSize = fontMetrics.width(formatBody);
                        formatBody = fontMetrics.elidedText(formatBody, Qt::ElideRight, oneFontSize - 1);
                        return formatBody;
                    }
                }
            } else {
                //说明只存在一行，在最后面加...就行
                formatBody = fontMetrics.elidedText(formatBody, Qt::ElideRight, LableWidth - 10);
                return formatBody;
            }
        }
        else if(PLATFORM::g_platformType == PLATFORM::Intel){  //Intel通知可显示两行
            if (list.size() >= 2) {
                //正文最多显示两行
                QString firstText;
                QString secondText;
                if (fontMetrics.width(list.at(0)) > (LableWidth - 10))
                {
                    firstText = fontMetrics.elidedText(list.at(0), Qt::ElideRight, LableWidth - 10);
                }else{
                    firstText = list.at(0);
                }
                if (fontMetrics.width(list.at(1)) > (LableWidth - 10)) {
                    secondText = fontMetrics.elidedText(list.at(1), Qt::ElideRight, LableWidth - 10);
                }else{
                    secondText = list.at(1);
                }
                formatBody = firstText+"\n"+secondText;
                //--<
            } else {
                //只有一行内容，换行显示，超出第二行的部分显示...
                QString strDisplay;
                //遍历正文字符串，在大于qlabel宽度的字符前加空格
                int j=1;
                for(int i=0;i<formatBody.length();i++){
                    strDisplay += QString(formatBody.at(i));
                    uint fontSize = fontMetrics.width(strDisplay);
                    if(fontSize > (j*(LableWidth-18))){
                        strDisplay+="\n ";
                        j++;
                    }
                }
                //超过2行末尾显示省略号
                formatBody = fontMetrics.elidedText(strDisplay, Qt::ElideRight, LableWidth*2 - 10);
                return formatBody;
            }
        }
    } else {
        QStringList list = formatBody.split("\n");
        if (list.size() >= 2) {
            //取得当前的有字符串的子串
            formatBody = judgeBlankLine(list);
            formatBody = formatBody + "aa";
            if (!substringSposition(formatBody, list)) {
                int oneFontSize = fontMetrics.width(formatBody);
                formatBody = fontMetrics.elidedText(formatBody, Qt::ElideRight, oneFontSize - 1);
            }
        }
    }
    return formatBody;
}

/* 去除掉空行，显示有字体的行 */
QString popupItemWidget::judgeBlankLine(QStringList list)
{
    int tmp = list.count();
    for (int i = 0; i < tmp; i++) {
        QString dest = list.at(i);
        dest = dest.trimmed();
        if (dest.size() != 0) {
           return list.at(i);
        }
    }
    return list.at(0);
}

/* 设置button字体长度，超出button时，设置... */
QString popupItemWidget::setButtonStringBody(QString text, QPushButton *button)
{
    QFontMetrics fontMetrics(button->font());
    int LableWidth = 90;
    int fontSize = fontMetrics.width(text);
    QString formatBody = text;
    if(fontSize > (LableWidth - 10)) {
        formatBody = fontMetrics.elidedText(formatBody, Qt::ElideRight, LableWidth - 10);
        return formatBody;
    }
    return formatBody;
}

/* 判断当前子串位置，后面是否还有子串 */
bool popupItemWidget::substringSposition(QString formatBody, QStringList list)
{
    int tmp = list.count();
    for (int i = 0; i < tmp; i++) {
        QString dest = list.at(i);
        if (dest == formatBody && i == tmp - 1) {
            return true;
        }
    }
    return false;
}

void popupItemWidget::jumpThroughSidebar(const QString &urlStr)
{
    QDBusMessage msg = QDBusMessage::createMethodCall("org.ukui.Sidebar",
                                                    "/org/ukui/Sidebar/notification",
                                                    "org.ukui.Sidebar.notification",
                                                    "proxyNotificationJump");
    msg << urlStr;
    QDBusConnection::sessionBus().asyncCall(msg);
}

/* 判断消息的标题是否存在 */
bool popupItemWidget::judgeSummaryExsit()
{
    if (m_pentryInfo->summary().isEmpty()) {
        if(PLATFORM::g_platformType == PLATFORM::V101)
            this->setFixedSize(372, 88);
        else if(PLATFORM::g_platformType == PLATFORM::Intel)
            this->setFixedSize(372, 110);
        return false;
    } else {
        return true;
    }
}

/* 判断消息体是否存在 */
bool popupItemWidget::judgeBodyExsit()
{
    if (m_pentryInfo->body().isEmpty()) {
        m_pBodyLabelWidget->setVisible(false);
        this->setFixedSize(372, 108);
        return false;
    } else {
        m_pBodyLabelWidget->setVisible(true);
        this->setFixedSize(372, 134);
        //解析正文字段
        processBody();
        return true;
    }
}

/* 判断图标是否存在 */
bool popupItemWidget::judgeIconExsit()
{
    if (m_pentryInfo->appIcon().isEmpty()) {
        return false;
    } else {
        return true;
    }
}

/* 判断action是否存在，存在则显示按钮 */
bool popupItemWidget::judgeActionExsit()
{
    if (m_pentryInfo->actions().isEmpty()) {
        processHints();  //为了兼容当下截图添加的兼容代码，等待截图改成标准协议，删掉即可！
        m_pOperationWidget->setVisible(false);
        if(PLATFORM::g_platformType == PLATFORM::V101)
            this->setFixedSize(372, 88);
        else if(PLATFORM::g_platformType == PLATFORM::Intel)
            this->setFixedSize(372, 110);
        return false;
    } else {
        // 解析m_actions链表，读取动作字符串名称和显示给用户的展示字符串
        processActions();
        return true;
    }
}

/* 解析正文字段的内容，将带有URI的消息体绑定动作，实现点击跳转 */
void popupItemWidget::processBody()
{
    //解析消息体中的数据:freedesktop 通知协议中规定，消息体正文字段可包含 XML、HTML以及一些附加标签
    QString bodyText;
    QString urlPath;
    QString msgBody = m_pentryInfo->body();
    if(msgBody.contains("href")){  //带有超链接的消息体单独处理，绑定信号槽
        if(msgBody.contains("wx") && msgBody.contains("qq.com")){ //网页微信发来的消息不太符合freedesktop通知协议,暂时单独分离出来
            //"<a href="https://wx2.qq.com/">wx2.qq.com</a>1"
            QXmlStreamReader xml(msgBody);
            while (!xml.atEnd()) {
                if (xml.tokenType() == QXmlStreamReader::StartElement){
                    QXmlStreamAttributes attributes = xml.attributes();
                    if(attributes.hasAttribute("href")){
                        urlPath = attributes.value("href").toString();
                    }
                }else if(xml.tokenType() == QXmlStreamReader::Characters){
                    bodyText = xml.text().toString();
                }
                xml.readNext();
            }
            //处理微信消息字段  <a href="https://wx.qq.com/">wx.qq.com</a> 111
            QStringList strList = msgBody.split("</a>");
            QString msg = strList.at(1);
            msg.remove(" ");
            msg.remove("\n");
            msg.remove("\r");
            m_pentryInfo->setBody(msg);

        }else{
            QXmlStreamReader xml(msgBody);
            while (!xml.atEnd()) {
                if (xml.tokenType() == QXmlStreamReader::StartElement){
                    QXmlStreamAttributes attributes = xml.attributes();
                    if(attributes.hasAttribute("href")){
                        urlPath = attributes.value("href").toString();
                    }
                }
                else if(xml.tokenType() == QXmlStreamReader::Characters){
                    bodyText = xml.text().toString();
                    m_pentryInfo->setBody(bodyText);
                }
                xml.readNext();
            }
        }
        if(m_pentryInfo->appIcon().contains("kylin-screenshot")){
            urlPath = "file://" + urlPath;
        }
        connect(this, &popupItemWidget::mouseMissed, this, [=](QWidget *w, int id){
            //通过侧边栏代理跳转Dbus接口完成跳转
            jumpThroughSidebar(urlPath);

        });
        if(!urlPath.isEmpty())
            m_pentryInfo->setUrlStr(urlPath);
    }
    else{ //普通消息暂时不做处理

    }

}

/* 解析动作字符串链表，添加动作按钮 */
void popupItemWidget::processActions()
{
    clearAllActionButton();
    QStringList list = m_pentryInfo->actions();
    if (list.contains("default")) {
        const int index  = list.indexOf("default"); // For the default action
        if(index>0){
            m_pDefaultAction = list[index-1];
            //有默认动作，绑定信号槽，点击弹窗触发
            connect(this, &popupItemWidget::actionInvokedMissed, this, [=](QWidget *w, int id){
                QProcess *process = new QProcess();
                process->start(m_pDefaultAction);
            });
            m_pentryInfo->setActionStr(m_pDefaultAction);
        }
        // Default action does not need to be displayed, removed from the list
        if(index % 2 == 0){  //为了兼容旧版协议
            list.removeAt(index + 1);
            list.removeAt(index);
        }else{              //新版协议
            list.removeAt(index);
            list.removeAt(index - 1);
        }
    }
    if (list.size() == 0) {
        m_pOperationWidget->setVisible(false);
        if(PLATFORM::g_platformType == PLATFORM::V101)
            this->setFixedSize(372, 88);
        else if(PLATFORM::g_platformType == PLATFORM::Intel)
            this->setFixedSize(372, 110);
    } else {
        m_pOperationWidget->setVisible(true);
        if(PLATFORM::g_platformType == PLATFORM::V101)
            this->setFixedSize(372, 134);
        else if(PLATFORM::g_platformType == PLATFORM::Intel)
            this->setFixedSize(372, 170);
        actionParsingJump(list);
    }
    return;
}

/* 解析按钮动作，绑定信号槽 */
void popupItemWidget::actionParsingJump(QStringList list)
{
    // Each even element in the list (starting at index 0) represents the
    // identifier for the action. Each odd element in the list is the
    // localized string that will be displayed to the user.
    if(PLATFORM::g_platformType == PLATFORM::Intel){
        /* 针对护眼中心页面跳转做的临时跳转方案（支持dbus跳转后会删除）：使用Dbus信号，两个动作对应两个信号 */
        if(m_pentryInfo->appName() == "护眼中心" && m_pentryInfo->summary() == "休息提醒"){
            qDebug()<<"护眼中心页面跳转 特殊处理"<<m_pentryInfo->actions().at(0)<<m_pentryInfo->actions().at(1);
            if(m_pentryInfo->actions().at(0) == "action1" && m_pentryInfo->actions().at(1) == "暂不休息"){
                qDebug()<<"护眼中心页面跳转  暂不休息  特殊处理";
                QPushButton *button = new QPushButton(m_pentryInfo->actions().at(1));
                m_pListButton->append(button);
                button->setFixedHeight(34);
                connect(button, &QPushButton::clicked, this, [=](){
                    //发射暂不休息DBus信号
                    QDBusMessage message =QDBusMessage::createSignal("/org/ukui/notify/eyeprotection",
                                                                     "org.ukui.notify.eyeprotection", "NoRest");
                    bool ret = QDBusConnection::sessionBus().send(message); //发射信号
                    if(ret)
                        qDebug() << "护眼中心按钮动作-暂不休息";
                    else
                        qDebug() << "护眼中心按钮动作-信号发送失败";
                    m_poutTimer->stop();
                    emit actionButtonClicked(this, m_pentryInfo->id().toInt(), m_pentryInfo->actions().at(0));
                });
                m_pOperationButtonWidgetLayout->addWidget(button, Qt::AlignRight);
                m_pOperationButtonWidgetLayout->setSpacing(10);
            }
            if(m_pentryInfo->actions().at(2) == "action2" && m_pentryInfo->actions().at(3) == "我知道了"){
                qDebug()<<"护眼中心页面跳转  知道了  特殊处理";
                QPushButton *button = new QPushButton(m_pentryInfo->actions().at(3));
                m_pListButton->append(button);
                button->setFixedHeight(34);
                connect(button, &QPushButton::clicked, this, [=](){
                    //发射暂不休息DBus信号
                    QDBusMessage message =QDBusMessage::createSignal("/org/ukui/notify/eyeprotection",
                                                                     "org.ukui.notify.eyeprotection", "GotIt");
                    bool ret = QDBusConnection::sessionBus().send(message); //发射信号
                    if(ret)
                        qDebug() << "护眼中心按钮动作-知道了";
                    else
                        qDebug() << "护眼中心按钮动作-信号发送失败";
                    m_poutTimer->stop();
                    emit actionButtonClicked(this, m_pentryInfo->id().toInt(), m_pentryInfo->actions().at(2));
                });
                m_pOperationButtonWidgetLayout->addWidget(button, Qt::AlignRight);
                m_pOperationButtonWidgetLayout->setSpacing(10);
            }
            return;
        }
    }

    //说白了，偶数是动作，奇数是要显示的名字
    //default关键字已做处理，当前list中不存在default和其对应的动作
    QString buttonAction;
    for (int i = 0; i != list.size(); ++i) {
        if (i % 2 == 0) {
            buttonAction  = list[i];
        } else {
            QString buttonText = list[i];
            QPushButton *button = new QPushButton();
            button->setStyleSheet("padding: 5px;}");
            m_pListButton->append(button);
            if(PLATFORM::g_platformType == PLATFORM::V101)
                button->setFixedHeight(36);
            else if(PLATFORM::g_platformType == PLATFORM::Intel)
                button->setFixedHeight(34);
            connect(button, &QPushButton::clicked, this, [=](){
                if (QProcess::startDetached(buttonAction)) {
                    qInfo() << "Button action successful";
                } else {
                    qCritical() << "Button action failed";
                }

                m_poutTimer->stop();
                emit actionButtonClicked(this, m_pentryInfo->id().toInt(), buttonAction);
            });
            m_pOperationButtonWidgetLayout->addWidget(button, Qt::AlignRight);
            m_pOperationButtonWidgetLayout->setSpacing(10);
            QString formatBody = setButtonStringBody(buttonText, button);
            button->setText(formatBody);
            connect(m_pFontStyleGsetting, &QGSettings::changed, this, [=](QString key) {
               if (key == "systemFontSize") {
                   button->setText(setButtonStringBody(buttonText, button));
               }
            });
        }
    }
}

/* 解析Map表,绑定弹窗跳转动作。兼容老协议，后期会去掉 */
void popupItemWidget::processHints()
{
    QMap<QString, QVariant> hints = m_pentryInfo->hints();
    QMap<QString, QVariant>::iterator iter = hints.begin();
    for(iter; iter!=hints.end(); iter++){
        if(iter.key() == URLS_ACTION){  //暂时为了兼容屏幕截图应用添加的代码，绑定URl链接，完成跳转
            QString urlPath = iter.value().toString();
            connect(this, &popupItemWidget::mouseMissed, this, [=](QWidget *w, int id){
                jumpThroughSidebar(urlPath);
            });
            if(!urlPath.isEmpty())
                m_pentryInfo->setUrlStr(urlPath);
        }
    }
}

void popupItemWidget::clearAllActionButton()
{
    for (QPushButton *button : *m_pListButton) {
        m_pOperationButtonWidgetLayout->removeWidget(button);
        button->deleteLater();
    }

    m_pListButton->clear();
    return;
}

/* 在通知弹窗上鼠标遗留一段时间后，退出notify */
void popupItemWidget::onDelayQuit()
{
    qApp->quit();
}

void popupItemWidget::resetMoveAnim(const QRect &rect)
{
    Q_UNUSED(rect);
    qDebug() << "重新设置动画";
}

//重新绘制背景色
void popupItemWidget::paintEvent(QPaintEvent *event)
{
    if(PLATFORM::g_platformType == PLATFORM::V101){
        QStyleOption opt;
        opt.init(this);
        QPainter p(this);
        /* 获取当前剪贴板中字体的颜色，作为背景色；
         * 白字体 --> 黑背景
         * 黑字体 --> 白字体
        */
        p.setBrush(opt.palette.color(QPalette::Base));
        p.setOpacity(m_fTransparencyValue);
        p.setPen(Qt::NoPen);
        QPainterPath path;
        opt.rect.adjust(0,0,0,0);
        path.addRoundedRect(opt.rect,6,6);
        p.setRenderHint(QPainter::Antialiasing);                        //反锯齿
        p.drawRoundedRect(opt.rect,6,6);
        setProperty("blurRegion",QRegion(path.toFillPolygon().toPolygon()));
        style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
        QWidget::paintEvent(event);
    }
    else if(PLATFORM::g_platformType == PLATFORM::Intel){
        QStyleOption opt;
            opt.init(this);
            QPainter p(this);
            /* 获取当前剪贴板中字体的颜色，作为背景色；
             * 白字体 --> 黑背景
             * 黑字体 --> 白字体
            */
            p.setBrush(opt.palette.color(QPalette::Base));
        //    p.setBrush(QBrush(QColor("#131314")));
            p.setOpacity(1);
            p.setPen(Qt::NoPen);
            QPainterPath path;
            opt.rect.adjust(0,0,0,0);
            path.addRoundedRect(opt.rect,12,12);
            p.setRenderHint(QPainter::Antialiasing);                        //反锯齿
            p.drawRoundedRect(opt.rect,12,12);
        //    setProperty("blurRegion",QRegion(path.toFillPolygon().toPolygon()));
        //    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
            QWidget::paintEvent(event);
    }
}

void popupItemWidget::mousePressEvent(QMouseEvent *event)
{
    // 点击消息体时，有默认动作则执行默认动作，无默认动作执行body字段中的链接
    if (!m_pDefaultAction.isEmpty()) {
        emit actionInvokedMissed(this, m_pentryInfo->id().toUInt(), m_pDefaultAction);
        m_pDefaultAction.clear();
    } else {
        if(m_pentryInfo->timeout() != "0"){ //非常驻应用点击消息体消失
            this->hide();
            emit mouseMissed(this, m_pentryInfo->id().toInt());
        }
    }
    m_poutTimer->stop();
    QWidget::mousePressEvent(event);
    return;
}

void popupItemWidget::showEvent(QShowEvent *event)
{
    QTimer::singleShot (1, this, [=] {
        raise();
    });
    m_quitTimer->start();
    QWidget::showEvent(event);
    return;
}

void popupItemWidget::hideEvent(QHideEvent *event)
{
    m_quitTimer->stop();
    QWidget::hideEvent(event);
    return;
}

/* 展示超时槽函数 */
void popupItemWidget::ShowTimeoutSlots()
{
    // 当鼠标在弹窗中时则会不关闭弹窗，显示时长为60分钟，当超过60分钟则会退出通知弹窗
    if (containsMouse()) {
        m_poutTimer->stop();
        m_poutTimer->start();
    } else {
        emit timeout(this);
    }
}

void popupItemWidget::qiutAppTimerSlots()
{
    qWarning() << "Killer Timeout, now quiiting...";
    qApp->quit();
}

void popupItemWidget::closeButtonSlots()
{
    this->hide();
    emit clickedMissed(this, m_pentryInfo->id().toInt());
    m_poutTimer->stop();
    return;
}

void popupItemWidget::OutAnimationFinishSlots()
{
    this->hide();
    emit timeOutMissed(this, m_pentryInfo->id().toInt());
    return;
}

void popupItemWidget::onActionButtonClicked(const QString &actionId)
{
//    QMap<QString, QVariant> hints = m_pentryInfo->hints();
//    QMap<QString, QVariant>::const_iterator i = hints.constBegin();
//    while (i != hints.constEnd()) {
//        QStringList args = i.value().toString().split(",");
//        if (!args.isEmpty()) {
//            QString cmd = args.first();
//            args.removeFirst();
//            if (i.key() == actionId) {
//                QProcess::startDetached(cmd, args);
//                if (QProcess::startDetached(cmd, args)) {
//                    qDebug() << "执行成功";
//                } else {
//                    qDebug() << "执行失败";
//                }
//            }
//        }
//        ++i;
//    }
//    m_poutTimer->stop();
//    emit actionInvokedMissed(this, m_pentryInfo->id().toInt(), actionId);
}
